﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.Windows.Threading;
using MindFusion.Charting.Wpf;
using System.Windows.Media.Media3D;
using System.Windows.Controls.Primitives;

namespace SurfaceChart
{
    /// <summary>
    /// Interaction logic for Window1.xaml
    /// </summary>
    public partial class Window1 : Window
    {
        private RadialGradientBrush brush;

        public Window1()
        {
            InitializeComponent();

            brush = new RadialGradientBrush(Colors.White, Colors.SteelBlue);
            brush.GradientOrigin = new Point(0.3, 0.3);
            brush.RadiusX = 0.7;
            brush.RadiusY = 0.7;


          
            Brush c = Brushes.LightSteelBlue;
            Brush b = Brushes.SteelBlue;
           

            SurfaceSeries series = new SurfaceSeries();
            series.Fills.Add(b);
            series.Strokes.Clear();
            series.Strokes.Add(c);
            series.ToolTipType = ToolTipType.ChartData;
            surfaceChart.Series.Add(series);

           

            surfaceChart.XAxisSettings.LabelFontSize = 11;
            surfaceChart.XAxisSettings.LabelOffset = 3;
            surfaceChart.XAxisSettings.LabelFormat = "0.##";
            surfaceChart.XAxisSettings.TitleFontSize = 11;
            surfaceChart.XAxisSettings.TitleOffset = 5;

            surfaceChart.YAxisSettings.LabelFontSize = 11;
            surfaceChart.YAxisSettings.LabelOffset = 3;
            surfaceChart.YAxisSettings.LabelFormat = "0.##";
            surfaceChart.YAxisSettings.TitleFontSize = 11;
            surfaceChart.YAxisSettings.TitleOffset = 5;
            surfaceChart.YAxisSettings.MinValue = -10;
            surfaceChart.YAxisSettings.MaxValue = 10;

            surfaceChart.ZAxisSettings.LabelFontSize = 11;
            surfaceChart.ZAxisSettings.LabelOffset = 3;
            surfaceChart.ZAxisSettings.LabelFormat = "0.##";
            surfaceChart.ZAxisSettings.TitleFontSize = 11;
            surfaceChart.ZAxisSettings.TitleOffset = 5;


            surfaceChart.SideWallThickness = 0.5;

            DirectionalLight light = new DirectionalLight()
            {
                Color = Colors.White,
                Direction = new Vector3D(-1, 3, -1),
            };
            surfaceChart.StaticLights.Add(light);
    
            RandomData(true);            
        }

        private Point3D AlignPoint3D(Point3D p)
        {
            double d = surfaceChart.PointMergeThreshold;
            double x = 30;
            double z = 30;
            if (p.X < d)
                p.X = 0;
            if (p.X > x - d)
                p.X = x;
            if (p.Z < d)
                p.Z = 0;
            if (p.Z > z - d)
                p.Z = z;
            return p;
        }

        private void RandomData(bool newElevations)
        {
            surfaceChart.BeginInit();

            surfaceChart.PointMergeThreshold = pointMergeThreshold.Value;
            surfaceChart.InterpolationSensitivity = sensitivity.Value;
            if (shape.SelectedIndex == 0)
                surfaceChart.InterpolationType = InterpolationType.Bezier;
            else if (shape.SelectedIndex == 1)
                surfaceChart.InterpolationType = InterpolationType.Linear;
            else
                surfaceChart.InterpolationType = InterpolationType.None;

            Random r = new Random();
            points.Clear();

            for (int i = 0; i < (int)randomData.Value; i++)
            {
                double x = r.NextDouble() * 30;
                double z = r.NextDouble() * 30;
                Point3D p = AlignPoint3D(new Point3D(x, 0, z));
                points.Add(p);                
            }
            points.Add(new Point3D(0, 0, 30));
            points.Add(new Point3D(30, 0, 30));
            points.Add(new Point3D(30, 0, 0));
            points.Add(new Point3D(0, 0, 0));

            if (newElevations)
            {
                elevations.Clear();
                for (int i = 0; i < 20; i++)
                    elevations.Add(new Point3D(r.Next(0, 30), r.Next(-10, 10), r.Next(0, 30)));
            }
            CreateElevations(points, elevations, surfaceChart.InterpolationSensitivity);

            foreach (SurfaceSeries s in surfaceChart.Series)
            {
                s.XData.Clear();
                s.YData.Clear();
                s.ZData.Clear();

                for (int i = 0; i < points.Count; i++)
                {
                    Point3D p = points[i];
                    s.XData.Add(p.X);
                    s.YData.Add(p.Y);
                    s.ZData.Add(p.Z);
                }   
            }

            surfaceChart.EndInit();
        }

        private void CreateElevations(List<Point3D> points,
            List<Point3D> elevations, double sensitivity)
        {
            double piCoef = Math.PI * 3 / 2;
            double rCoef = 10 / Math.PI;

            for (int j = 0; j < elevations.Count; j++)
            {
                double dataX = elevations[j].X;
                double dataZ = elevations[j].Z;
                double dataY = elevations[j].Y;

                for (int i = 0; i < points.Count; i++)
                {
                    Point3D p = points[i];

                    //Corners
                    double dx = dataX - p.X;
                    double dz = dataZ - p.Z;
                    double distance = Math.Sqrt(dx * dx + dz * dz);

                    if (distance <= sensitivity)
                    {
                        double percent = 1.0 - distance / sensitivity;
                        double coef = dataY - p.Y;

                        double elevate = coef * (Math.Sin(percent * rCoef + piCoef) + 1.0) / 2;
                        p.Y += elevate;
                        points[i] = p;
                    }
                }
            }
        }

        private void button2_Click(object sender, RoutedEventArgs e)
        {
            RandomData(true);
        }

        private void viewport3D_MouseUp(object sender, MouseButtonEventArgs e)
        {
            surfaceChart.ReleaseMouseCapture();
        }

        private void CheckBoxChecked(object sender, RoutedEventArgs e)
        {
            if (surfaceChart == null)
                return;

            CheckBox box = sender as CheckBox;
            if (box.Name == "showSurface")
                surfaceChart.ShowSurface = true;
            if (box.Name == "showScatters")
                surfaceChart.ShowScatters = true;
            if (box.Name == "showWireFrame")
                surfaceChart.ShowWireFrame = true;
            if (box.Name == "showXAxis")
                surfaceChart.XAxisSettings.LabelType = LabelType.AutoScale;
            if (box.Name == "showYAxis")
                surfaceChart.YAxisSettings.LabelType = LabelType.AutoScale;
            if (box.Name == "showZAxis")
                surfaceChart.ZAxisSettings.LabelType = LabelType.AutoScale;
            if (box.Name == "closeModelFigure")
                surfaceChart.CloseModelFigure = true;
        }

        private void CheckBoxUnchecked(object sender, RoutedEventArgs e)
        {
            CheckBox box = sender as CheckBox;
            if (box.Name == "showSurface")
                surfaceChart.ShowSurface = false;
            if (box.Name == "showScatters")
                surfaceChart.ShowScatters = false;
            if (box.Name == "showWireFrame")
                surfaceChart.ShowWireFrame = false;
            if (box.Name == "showXAxis")
                surfaceChart.XAxisSettings.LabelType = LabelType.Empty;
            if (box.Name == "showYAxis")
                surfaceChart.YAxisSettings.LabelType = LabelType.Empty;
            if (box.Name == "showZAxis")
                surfaceChart.ZAxisSettings.LabelType = LabelType.Empty;
            if (box.Name == "closeModelFigure")
                surfaceChart.CloseModelFigure = false;
        }

        private void ComboBoxSelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            if (surfaceChart == null)
                return;

            ComboBox box = sender as ComboBox;
            if (box.Name == "cameraType")
            {
                surfaceChart.CameraType = box.SelectedIndex == 0 ?
                    CameraType.Perspective : CameraType.Orthographic;
            }
            if (box.Name == "shape")
            {
                if (box.SelectedIndex == 0)
                    surfaceChart.InterpolationType = InterpolationType.Bezier;
                else if (box.SelectedIndex == 1)
                    surfaceChart.InterpolationType = InterpolationType.Linear;
                else
                    RandomData(false);
            }
            if (box.Name == "textureType")
            {
                surfaceChart.TextureType = box.SelectedIndex == 0 ?
                    TextureType.Brush : TextureType.GradientColorMap;
            }            
            if (box.Name == "effect3D")
            {
                surfaceChart.Effect3D = box.SelectedIndex == 0 ?
                   ChartEffect3D.None : ChartEffect3D.ShaderEffect;
            }
        }

        private void flatSurface_Click(object sender, RoutedEventArgs e)
        {
            (surfaceChart.Series[0] as SurfaceSeries).XData.Clear();
            (surfaceChart.Series[0] as SurfaceSeries).YData.Clear();
            (surfaceChart.Series[0] as SurfaceSeries).ZData.Clear();
        }

        private void SliderValueChanged(object sender, RoutedPropertyChangedEventArgs<double> e)
        {
            if (surfaceChart == null)
                return;

            Slider slider = sender as Slider;
            if (slider.Name == "squireSize")
                surfaceChart.MeshFaceSize = e.NewValue;
            if (slider.Name == "sensitivity")
            {
                if (surfaceChart.InterpolationType == InterpolationType.None)
                    RandomData(false);
                else
                    surfaceChart.InterpolationSensitivity = e.NewValue;            
            }
            if (slider.Name == "groundLevel")
                surfaceChart.GroundLevel = e.NewValue;
            if (slider.Name == "wireFrameThickness")
                surfaceChart.WireFrameThickness = e.NewValue;
        }      

        private void pointMergeThreshold_PreviewMouseUp(object sender, MouseButtonEventArgs e)
        {
            if (surfaceChart.InterpolationType == InterpolationType.None)
                RandomData(false);
            else
                surfaceChart.PointMergeThreshold = pointMergeThreshold.Value;
        }

        private List<Point3D> points = new List<Point3D>();
        private List<Point3D> elevations = new List<Point3D>();
    }
}
